from enum import IntEnum
import struct
from typing import Literal

from google.protobuf.json_format import MessageToDict, Parse
from pydantic import BaseModel

from .pb.command_pb2 import (
    PHeartBeat,  # type: ignore
    PHeartBeatReply,  # type: ignore
    PKickOff,  # type: ignore
    PLogin,  # type: ignore
    PLoginReply,  # type: ignore
    PLogout,  # type: ignore
    PLogoutReply,  # type: ignore
)
from .pb.model_pb2 import RobotEvent as PRobotEvent  # type: ignore


class BizType(IntEnum):
    UNKNOWN = 0
    EXCHANGE_KEY = 1
    HEARTBEAT = 2
    LOGIN = 3
    LOGOUT = 4
    P_EXCHANGE_KEY = 5
    P_HEARTBEAT = 6
    P_LOGIN = 7
    P_LOGOUT = 8
    KICK_OFF = 51
    SHUTDOWN = 52
    P_KICK_OFF = 53
    ROOM_ENTER = 60
    ROOM_LEAVE = 61
    ROOM_CLOSE = 62
    ROOM_MSG = 63
    EVENT = 30001


class Payload(BaseModel):
    id: int
    flag: Literal[1, 2]
    biz_type: BizType
    app_id: Literal[104] = 104
    body_data: bytes

    @classmethod
    def from_bytes(cls, data: bytes):
        magic, data_len = struct.unpack("<II", data[:8])
        header_len, packet_id, flag, biz_type, app_id = struct.unpack(
            "<IQIIi",
            data[8:32],
        )
        body_data = data[32 : 8 + data_len]
        return cls(
            id=packet_id,
            flag=flag,
            biz_type=BizType(biz_type),
            app_id=app_id,
            body_data=body_data,
        )

    def to_bytes(self) -> bytes:
        changeable = (
            struct.pack(
                "<IQIIi",
                24,
                self.id,
                self.flag,
                self.biz_type,
                self.app_id,
            )
            + self.body_data
        )

        return (
            struct.pack(
                "<II",
                0xBABEFACE,
                len(changeable),
            )
            + changeable
        )


class HeartBeat(BaseModel):
    """心跳请求命令字"""

    # 客户端时间戳，精确到ms
    client_timestamp: str

    def to_bytes_package(self, id: int) -> bytes:
        return Payload(
            id=id,
            flag=1,
            biz_type=BizType.P_HEARTBEAT,
            body_data=Parse(self.json(), PHeartBeat()).SerializeToString(),
        ).to_bytes()


class HeartBeatReply(BaseModel):
    """心跳返回"""

    # 错误码 非0表示失败
    code: int = 0
    # 服务端时间戳，精确到ms
    server_timestamp: int

    @classmethod
    def from_proto(cls, content: bytes) -> "HeartBeatReply":
        return cls.parse_obj(
            MessageToDict(
                PHeartBeatReply().FromString(content),
                preserving_proto_field_name=True,
                use_integers_for_enums=True,
            ),
        )


class Login(BaseModel):
    """登录命令"""

    # 长连接侧唯一id，uint64格式
    uid: int
    # 用于业务后端验证的token
    token: str
    # 客户端操作平台枚举
    platform: int
    # 业务所在客户端应用标识，用于在同一个客户端隔离不同业务的长连接通道。
    app_id: int
    device_id: str
    # # 区域划分字段，通过uid+app_id+platform+region四个字段唯一确定一条长连接
    ## region: str
    # # 长连内部的扩展字段，是个map
    # meta: Dict[str, str]

    def to_bytes_package(self, id: int) -> bytes:
        return Payload(
            id=id,
            flag=1,
            biz_type=BizType.P_LOGIN,
            body_data=Parse(self.json(), PLogin()).SerializeToString(),
        ).to_bytes()


class LoginReply(BaseModel):
    """登录命令返回"""

    # 错误码 非0表示失败
    code: int = 0
    # 错误信息
    msg: str = ""
    # 服务端时间戳，精确到ms
    server_timestamp: int
    # 唯一连接ID
    conn_id: int

    @classmethod
    def from_proto(cls, content: bytes) -> "LoginReply":
        return cls.parse_obj(
            MessageToDict(
                PLoginReply().FromString(content),
                preserving_proto_field_name=True,
                use_integers_for_enums=True,
            ),
        )


class Logout(BaseModel):
    """登出命令字"""

    # 长连接侧唯一id，uint64格式
    uid: int
    # 客户端操作平台枚举
    platform: int
    # 业务所在客户端应用标识，用于在同一个客户端隔离不同业务的长连接通道。
    app_id: int
    # 客户端设备唯一标识
    device_id: str
    # 区域划分字段，通过uid+app_id+platform+region四个字段唯一确定一条长连接
    ## region: str

    def to_bytes_package(self, id: int) -> bytes:
        return Payload(
            id=id,
            flag=1,
            biz_type=BizType.P_LOGOUT,
            body_data=Parse(self.json(), PLogout()).SerializeToString(),
        ).to_bytes()


class LogoutReply(BaseModel):
    """登出命令返回"""

    # 错误码 非0表示失败
    code: int = 0
    # 错误信息
    msg: str = ""
    # 连接id
    conn_id: int

    @classmethod
    def from_proto(cls, content: bytes) -> "LogoutReply":
        return cls.parse_obj(
            MessageToDict(
                PLogoutReply().FromString(content),
                preserving_proto_field_name=True,
                use_integers_for_enums=True,
            ),
        )


class KickOff(BaseModel):
    """踢出连接协议"""

    # 踢出原因状态码
    code: int = 0
    # 状态码对应的文案
    reason: str = ""

    @classmethod
    def from_proto(cls, content: bytes) -> "KickOff":
        return cls.parse_obj(
            MessageToDict(
                PKickOff().FromString(content),
                preserving_proto_field_name=True,
                use_integers_for_enums=True,
            ),
        )


class Shutdown(BaseModel):
    """服务关机"""


def proto_to_event_data(content: bytes):
    return MessageToDict(
        PRobotEvent().FromString(content),
        preserving_proto_field_name=True,
        use_integers_for_enums=True,
    )



from functools import partial
import hashlib
import imghdr
import re
from typing import (
    TYPE_CHECKING,
    Any,
    Awaitable,
    Callable,
    Dict,
    Generic,
    Optional,
    Type,
    TypeVar,
    overload,
)
from typing_extensions import Concatenate, ParamSpec

from nonebot.utils import logger_wrapper

if TYPE_CHECKING:
    from .bot import Bot

B = TypeVar("B", bound="Bot")
R = TypeVar("R")
P = ParamSpec("P")


log = logger_wrapper("Villa")


def pascal_to_snake(string):
    result = string[0].lower()

    pattern = re.compile(r"(?<!^)(?=[A-Z])")
    result += pattern.sub("_", string[1:]).lower()

    return result


def exclude_none(data: Dict[str, Any]) -> Dict[str, Any]:
    return {k: v for k, v in data.items() if v is not None}


class API(Generic[B, P, R]):
    def __init__(self, func: Callable[Concatenate[B, P], Awaitable[R]]) -> None:
        self.func = func

    def __set_name__(self, owner: Type[B], name: str) -> None:
        self.name = name

    @overload
    def __get__(self, obj: None, objtype: Type[B]) -> "API[B, P, R]":
        ...

    @overload
    def __get__(self, obj: B, objtype: Optional[Type[B]]) -> Callable[P, Awaitable[R]]:
        ...

    def __get__(
        self,
        obj: Optional[B],
        objtype: Optional[Type[B]] = None,
    ) -> "API[B, P, R] | Callable[P, Awaitable[R]]":
        if obj is None:
            return self

        return partial(obj.call_api, self.name)  # type: ignore

    async def __call__(self, inst: B, *args: P.args, **kwds: P.kwargs) -> R:
        return await self.func(inst, *args, **kwds)


def get_img_extenion(img_bytes: bytes) -> Optional[str]:
    return imghdr.what(None, h=img_bytes)


def get_img_md5(img_bytes: bytes) -> str:
    return hashlib.md5(img_bytes).hexdigest()



from .adapter import Adapter as Adapter
from .bot import Bot as Bot
from .event import *
from .exception import ActionFailed as ActionFailed
from .message import (
    Message as Message,
    MessageSegment as MessageSegment,
)



from typing import Union

from nonebot.adapters.villa import AddQuickEmoticonEvent, Bot, SendMessageEvent
from nonebot.adapters.villa.api import RoleType
from nonebot.permission import Permission


async def is_owner_or_admin(
    bot: Bot,
    event: Union[SendMessageEvent, AddQuickEmoticonEvent],
) -> bool:
    user_id = event.from_user_id if isinstance(event, SendMessageEvent) else event.uid
    user = await bot.get_member(villa_id=event.villa_id, uid=user_id)
    return any(
        role.role_type in (RoleType.OWNER, RoleType.ADMIN) for role in user.role_list
    )


OWNER_OR_ADMIN = Permission(is_owner_or_admin)
"""别野房东或管理员权限"""


async def is_owner(
    bot: Bot,
    event: Union[SendMessageEvent, AddQuickEmoticonEvent],
) -> bool:
    user_id = event.from_user_id if isinstance(event, SendMessageEvent) else event.uid
    user = await bot.get_member(villa_id=event.villa_id, uid=user_id)
    return any(role.role_type == RoleType.OWNER for role in user.role_list)


OWNER = Permission(is_owner)
"""别野房东权限"""


async def is_admin(
    bot: Bot,
    event: Union[SendMessageEvent, AddQuickEmoticonEvent],
) -> bool:
    user_id = event.from_user_id if isinstance(event, SendMessageEvent) else event.uid
    user = await bot.get_member(villa_id=event.villa_id, uid=user_id)
    return any(role.role_type == RoleType.ADMIN for role in user.role_list)


ADMIN = Permission(is_admin)
"""管理员权限"""



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: command.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""
from google.protobuf import (
    descriptor as _descriptor,
    descriptor_pool as _descriptor_pool,
    symbol_database as _symbol_database,
)
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\rcommand.proto\x12\x08vila_bot"&\n\nPHeartBeat\x12\x18\n\x10\x63lient_timestamp\x18\x01 \x01(\t"9\n\x0fPHeartBeatReply\x12\x0c\n\x04\x63ode\x18\x01 \x01(\x05\x12\x18\n\x10server_timestamp\x18\x02 \x01(\x04"\xc0\x01\n\x06PLogin\x12\x0b\n\x03uid\x18\x01 \x01(\x04\x12\r\n\x05token\x18\x02 \x01(\t\x12\x10\n\x08platform\x18\x03 \x01(\x05\x12\x0e\n\x06\x61pp_id\x18\x04 \x01(\x05\x12\x11\n\tdevice_id\x18\x05 \x01(\t\x12\x0e\n\x06region\x18\x06 \x01(\t\x12(\n\x04meta\x18\x07 \x03(\x0b\x32\x1a.vila_bot.PLogin.MetaEntry\x1a+\n\tMetaEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01"S\n\x0bPLoginReply\x12\x0c\n\x04\x63ode\x18\x01 \x01(\x05\x12\x0b\n\x03msg\x18\x02 \x01(\t\x12\x18\n\x10server_timestamp\x18\x03 \x01(\x04\x12\x0f\n\x07\x63onn_id\x18\x04 \x01(\x04"[\n\x07PLogout\x12\x0b\n\x03uid\x18\x01 \x01(\x04\x12\x10\n\x08platform\x18\x02 \x01(\x05\x12\x0e\n\x06\x61pp_id\x18\x03 \x01(\x05\x12\x11\n\tdevice_id\x18\x04 \x01(\t\x12\x0e\n\x06region\x18\x05 \x01(\t":\n\x0cPLogoutReply\x12\x0c\n\x04\x63ode\x18\x01 \x01(\x05\x12\x0b\n\x03msg\x18\x02 \x01(\t\x12\x0f\n\x07\x63onn_id\x18\x03 \x01(\x04"(\n\x0b\x43ommonReply\x12\x0c\n\x04\x63ode\x18\x01 \x01(\x05\x12\x0b\n\x03msg\x18\x02 \x01(\t"(\n\x08PKickOff\x12\x0c\n\x04\x63ode\x18\x01 \x01(\x05\x12\x0e\n\x06reason\x18\x02 \x01(\t*\xf8\x01\n\x07\x43ommand\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x10\n\x0c\x45XCHANGE_KEY\x10\x01\x12\r\n\tHEARTBEAT\x10\x02\x12\t\n\x05LOGIN\x10\x03\x12\n\n\x06LOGOUT\x10\x04\x12\x12\n\x0eP_EXCHANGE_KEY\x10\x05\x12\x0f\n\x0bP_HEARTBEAT\x10\x06\x12\x0b\n\x07P_LOGIN\x10\x07\x12\x0c\n\x08P_LOGOUT\x10\x08\x12\x0c\n\x08KICK_OFF\x10\x33\x12\x0c\n\x08SHUTDOWN\x10\x34\x12\x0e\n\nP_KICK_OFF\x10\x35\x12\x0e\n\nROOM_ENTER\x10<\x12\x0e\n\nROOM_LEAVE\x10=\x12\x0e\n\nROOM_CLOSE\x10>\x12\x0c\n\x08ROOM_MSG\x10?B6Z4gopkg.mihoyo.com/vila-bot-go/proto/vila_bot;vila_botb\x06proto3',  # noqa: E501
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "command_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS is False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"Z4gopkg.mihoyo.com/vila-bot-go/proto/vila_bot;vila_bot"
    _globals["_PLOGIN_METAENTRY"]._options = None
    _globals["_PLOGIN_METAENTRY"]._serialized_options = b"8\001"
    _globals["_COMMAND"]._serialized_start = 644
    _globals["_COMMAND"]._serialized_end = 892
    _globals["_PHEARTBEAT"]._serialized_start = 27
    _globals["_PHEARTBEAT"]._serialized_end = 65
    _globals["_PHEARTBEATREPLY"]._serialized_start = 67
    _globals["_PHEARTBEATREPLY"]._serialized_end = 124
    _globals["_PLOGIN"]._serialized_start = 127
    _globals["_PLOGIN"]._serialized_end = 319
    _globals["_PLOGIN_METAENTRY"]._serialized_start = 276
    _globals["_PLOGIN_METAENTRY"]._serialized_end = 319
    _globals["_PLOGINREPLY"]._serialized_start = 321
    _globals["_PLOGINREPLY"]._serialized_end = 404
    _globals["_PLOGOUT"]._serialized_start = 406
    _globals["_PLOGOUT"]._serialized_end = 497
    _globals["_PLOGOUTREPLY"]._serialized_start = 499
    _globals["_PLOGOUTREPLY"]._serialized_end = 557
    _globals["_COMMONREPLY"]._serialized_start = 559
    _globals["_COMMONREPLY"]._serialized_end = 599
    _globals["_PKICKOFF"]._serialized_start = 601
    _globals["_PKICKOFF"]._serialized_end = 641
# @@protoc_insertion_point(module_scope)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: model.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""
from google.protobuf import (
    descriptor as _descriptor,
    descriptor_pool as _descriptor_pool,
    symbol_database as _symbol_database,
)
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x0bmodel.proto\x12\x08vila_bot"\xf8\x02\n\rRobotTemplate\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x0c\n\x04\x64\x65sc\x18\x03 \x01(\t\x12\x0c\n\x04icon\x18\x04 \x01(\t\x12\x31\n\x08\x63ommands\x18\x05 \x03(\x0b\x32\x1f.vila_bot.RobotTemplate.Command\x12>\n\x0f\x63ustom_settings\x18\x06 \x03(\x0b\x32%.vila_bot.RobotTemplate.CustomSetting\x12%\n\x1dis_allowed_add_to_other_villa\x18\x07 \x01(\x08\x1a\x15\n\x05Param\x12\x0c\n\x04\x64\x65sc\x18\x01 \x01(\t\x1aT\n\x07\x43ommand\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0c\n\x04\x64\x65sc\x18\x02 \x01(\t\x12-\n\x06params\x18\x03 \x03(\x0b\x32\x1d.vila_bot.RobotTemplate.Param\x1a*\n\rCustomSetting\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0b\n\x03url\x18\x02 \x01(\t"D\n\x05Robot\x12)\n\x08template\x18\x01 \x01(\x0b\x32\x17.vila_bot.RobotTemplate\x12\x10\n\x08villa_id\x18\x02 \x01(\x04"\xb7\x01\n\x10QuoteMessageInfo\x12\x0f\n\x07\x63ontent\x18\x01 \x01(\t\x12\x0f\n\x07msg_uid\x18\x02 \x01(\t\x12\x0f\n\x07send_at\x18\x03 \x01(\x03\x12\x10\n\x08msg_type\x18\x04 \x01(\t\x12\x12\n\nbot_msg_id\x18\x05 \x01(\t\x12\x14\n\x0c\x66rom_user_id\x18\x06 \x01(\x04\x12\x18\n\x10\x66rom_user_id_str\x18\x07 \x01(\t\x12\x1a\n\x12\x66rom_user_nickname\x18\x08 \x01(\t"\xbd\x0f\n\nRobotEvent\x12\x1e\n\x05robot\x18\x01 \x01(\x0b\x32\x0f.vila_bot.Robot\x12,\n\x04type\x18\x02 \x01(\x0e\x32\x1e.vila_bot.RobotEvent.EventType\x12\x34\n\x0b\x65xtend_data\x18\x03 \x01(\x0b\x32\x1f.vila_bot.RobotEvent.ExtendData\x12\x12\n\ncreated_at\x18\x04 \x01(\x03\x12\n\n\x02id\x18\x05 \x01(\t\x12\x0f\n\x07send_at\x18\x06 \x01(\x03\x1a\xcf\x0c\n\nExtendData\x12\x43\n\njoin_villa\x18\x01 \x01(\x0b\x32-.vila_bot.RobotEvent.ExtendData.JoinVillaInfoH\x00\x12G\n\x0csend_message\x18\x02 \x01(\x0b\x32/.vila_bot.RobotEvent.ExtendData.SendMessageInfoH\x00\x12G\n\x0c\x63reate_robot\x18\x03 \x01(\x0b\x32/.vila_bot.RobotEvent.ExtendData.CreateRobotInfoH\x00\x12G\n\x0c\x64\x65lete_robot\x18\x04 \x01(\x0b\x32/.vila_bot.RobotEvent.ExtendData.DeleteRobotInfoH\x00\x12R\n\x12\x61\x64\x64_quick_emoticon\x18\x05 \x01(\x0b\x32\x34.vila_bot.RobotEvent.ExtendData.AddQuickEmoticonInfoH\x00\x12K\n\x0e\x61udit_callback\x18\x06 \x01(\x0b\x32\x31.vila_bot.RobotEvent.ExtendData.AuditCallbackInfoH\x00\x12T\n\x13\x63lick_msg_component\x18\x07 \x01(\x0b\x32\x35.vila_bot.RobotEvent.ExtendData.ClickMsgComponentInfoH\x00\x1a`\n\rJoinVillaInfo\x12\x10\n\x08join_uid\x18\x01 \x01(\x04\x12\x1a\n\x12join_user_nickname\x18\x02 \x01(\t\x12\x0f\n\x07join_at\x18\x03 \x01(\x03\x12\x10\n\x08villa_id\x18\x04 \x01(\x04\x1a\xfd\x01\n\x0fSendMessageInfo\x12\x0f\n\x07\x63ontent\x18\x01 \x01(\t\x12\x14\n\x0c\x66rom_user_id\x18\x02 \x01(\x04\x12\x0f\n\x07send_at\x18\x03 \x01(\x03\x12)\n\x0bobject_name\x18\x04 \x01(\x0e\x32\x14.vila_bot.ObjectName\x12\x0f\n\x07room_id\x18\x05 \x01(\x04\x12\x10\n\x08nickname\x18\x06 \x01(\t\x12\x0f\n\x07msg_uid\x18\x07 \x01(\t\x12\x12\n\nbot_msg_id\x18\x08 \x01(\t\x12\x10\n\x08villa_id\x18\t \x01(\x04\x12-\n\tquote_msg\x18\n \x01(\x0b\x32\x1a.vila_bot.QuoteMessageInfo\x1a#\n\x0f\x43reateRobotInfo\x12\x10\n\x08villa_id\x18\x01 \x01(\x04\x1a#\n\x0f\x44\x65leteRobotInfo\x12\x10\n\x08villa_id\x18\x01 \x01(\x04\x1a\xbc\x01\n\x14\x41\x64\x64QuickEmoticonInfo\x12\x10\n\x08villa_id\x18\x01 \x01(\x04\x12\x0f\n\x07room_id\x18\x02 \x01(\x04\x12\x0b\n\x03uid\x18\x03 \x01(\x04\x12\x13\n\x0b\x65moticon_id\x18\x04 \x01(\r\x12\x10\n\x08\x65moticon\x18\x05 \x01(\t\x12\x0f\n\x07msg_uid\x18\x06 \x01(\t\x12\x11\n\tis_cancel\x18\x07 \x01(\x08\x12\x12\n\nbot_msg_id\x18\x08 \x01(\t\x12\x15\n\remoticon_type\x18\t \x01(\r\x1a\x87\x02\n\x11\x41uditCallbackInfo\x12\x10\n\x08\x61udit_id\x18\x01 \x01(\t\x12\x12\n\nbot_tpl_id\x18\x02 \x01(\t\x12\x10\n\x08villa_id\x18\x03 \x01(\x04\x12\x0f\n\x07room_id\x18\x04 \x01(\x04\x12\x0f\n\x07user_id\x18\x05 \x01(\x04\x12\x14\n\x0cpass_through\x18\x06 \x01(\t\x12S\n\x0c\x61udit_result\x18\x07 \x01(\x0e\x32=.vila_bot.RobotEvent.ExtendData.AuditCallbackInfo.AuditResult"-\n\x0b\x41uditResult\x12\x08\n\x04None\x10\x00\x12\x08\n\x04Pass\x10\x01\x12\n\n\x06Reject\x10\x02\x1a\xa6\x01\n\x15\x43lickMsgComponentInfo\x12\x10\n\x08villa_id\x18\x01 \x01(\x04\x12\x0f\n\x07room_id\x18\x02 \x01(\x04\x12\x14\n\x0c\x63omponent_id\x18\x03 \x01(\t\x12\x0f\n\x07msg_uid\x18\x04 \x01(\t\x12\x0b\n\x03uid\x18\x05 \x01(\x04\x12\x12\n\nbot_msg_id\x18\x06 \x01(\t\x12\x13\n\x0btemplate_id\x18\x07 \x01(\x04\x12\r\n\x05\x65xtra\x18\x08 \x01(\tB\x0c\n\nevent_data"\xa7\x01\n\tEventType\x12\x18\n\x14UnknowRobotEventType\x10\x00\x12\r\n\tJoinVilla\x10\x01\x12\x0f\n\x0bSendMessage\x10\x02\x12\x0f\n\x0b\x43reateRobot\x10\x03\x12\x0f\n\x0b\x44\x65leteRobot\x10\x04\x12\x14\n\x10\x41\x64\x64QuickEmoticon\x10\x05\x12\x11\n\rAuditCallback\x10\x06\x12\x15\n\x11\x43lickMsgComponent\x10\x07*b\n\x08RoomType\x12\x13\n\x0fRoomTypeInvalid\x10\x00\x12\x14\n\x10RoomTypeChatRoom\x10\x01\x12\x14\n\x10RoomTypePostRoom\x10\x02\x12\x15\n\x11RoomTypeSceneRoom\x10\x03*6\n\nObjectName\x12\x14\n\x10UnknowObjectName\x10\x00\x12\x08\n\x04Text\x10\x01\x12\x08\n\x04Post\x10\x02\x42\x36Z4gopkg.mihoyo.com/vila-bot-go/proto/vila_bot;vila_botb\x06proto3',  # noqa: E501
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "model_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS is False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"Z4gopkg.mihoyo.com/vila-bot-go/proto/vila_bot;vila_bot"
    _globals["_ROOMTYPE"]._serialized_start = 2644
    _globals["_ROOMTYPE"]._serialized_end = 2742
    _globals["_OBJECTNAME"]._serialized_start = 2744
    _globals["_OBJECTNAME"]._serialized_end = 2798
    _globals["_ROBOTTEMPLATE"]._serialized_start = 26
    _globals["_ROBOTTEMPLATE"]._serialized_end = 402
    _globals["_ROBOTTEMPLATE_PARAM"]._serialized_start = 251
    _globals["_ROBOTTEMPLATE_PARAM"]._serialized_end = 272
    _globals["_ROBOTTEMPLATE_COMMAND"]._serialized_start = 274
    _globals["_ROBOTTEMPLATE_COMMAND"]._serialized_end = 358
    _globals["_ROBOTTEMPLATE_CUSTOMSETTING"]._serialized_start = 360
    _globals["_ROBOTTEMPLATE_CUSTOMSETTING"]._serialized_end = 402
    _globals["_ROBOT"]._serialized_start = 404
    _globals["_ROBOT"]._serialized_end = 472
    _globals["_QUOTEMESSAGEINFO"]._serialized_start = 475
    _globals["_QUOTEMESSAGEINFO"]._serialized_end = 658
    _globals["_ROBOTEVENT"]._serialized_start = 661
    _globals["_ROBOTEVENT"]._serialized_end = 2642
    _globals["_ROBOTEVENT_EXTENDDATA"]._serialized_start = 857
    _globals["_ROBOTEVENT_EXTENDDATA"]._serialized_end = 2472
    _globals["_ROBOTEVENT_EXTENDDATA_JOINVILLAINFO"]._serialized_start = 1406
    _globals["_ROBOTEVENT_EXTENDDATA_JOINVILLAINFO"]._serialized_end = 1502
    _globals["_ROBOTEVENT_EXTENDDATA_SENDMESSAGEINFO"]._serialized_start = 1505
    _globals["_ROBOTEVENT_EXTENDDATA_SENDMESSAGEINFO"]._serialized_end = 1758
    _globals["_ROBOTEVENT_EXTENDDATA_CREATEROBOTINFO"]._serialized_start = 1760
    _globals["_ROBOTEVENT_EXTENDDATA_CREATEROBOTINFO"]._serialized_end = 1795
    _globals["_ROBOTEVENT_EXTENDDATA_DELETEROBOTINFO"]._serialized_start = 1797
    _globals["_ROBOTEVENT_EXTENDDATA_DELETEROBOTINFO"]._serialized_end = 1832
    _globals["_ROBOTEVENT_EXTENDDATA_ADDQUICKEMOTICONINFO"]._serialized_start = 1835
    _globals["_ROBOTEVENT_EXTENDDATA_ADDQUICKEMOTICONINFO"]._serialized_end = 2023
    _globals["_ROBOTEVENT_EXTENDDATA_AUDITCALLBACKINFO"]._serialized_start = 2026
    _globals["_ROBOTEVENT_EXTENDDATA_AUDITCALLBACKINFO"]._serialized_end = 2289
    _globals[
        "_ROBOTEVENT_EXTENDDATA_AUDITCALLBACKINFO_AUDITRESULT"
    ]._serialized_start = 2244
    _globals[
        "_ROBOTEVENT_EXTENDDATA_AUDITCALLBACKINFO_AUDITRESULT"
    ]._serialized_end = 2289
    _globals["_ROBOTEVENT_EXTENDDATA_CLICKMSGCOMPONENTINFO"]._serialized_start = 2292
    _globals["_ROBOTEVENT_EXTENDDATA_CLICKMSGCOMPONENTINFO"]._serialized_end = 2458
    _globals["_ROBOTEVENT_EVENTTYPE"]._serialized_start = 2475
    _globals["_ROBOTEVENT_EVENTTYPE"]._serialized_end = 2642
# @@protoc_insertion_point(module_scope)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: robot_event_message.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""
from google.protobuf import (
    descriptor as _descriptor,
    descriptor_pool as _descriptor_pool,
    symbol_database as _symbol_database,
)
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x19robot_event_message.proto\x12\x08vila_bot\x1a\x0bmodel.proto"8\n\x11RobotEventMessage\x12#\n\x05\x65vent\x18\x01 \x01(\x0b\x32\x14.vila_bot.RobotEventB6Z4gopkg.mihoyo.com/vila-bot-go/proto/vila_bot;vila_botb\x06proto3',  # noqa: E501
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "robot_event_message_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS is False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"Z4gopkg.mihoyo.com/vila-bot-go/proto/vila_bot;vila_bot"
    _globals["_ROBOTEVENTMESSAGE"]._serialized_start = 52
    _globals["_ROBOTEVENTMESSAGE"]._serialized_end = 108
# @@protoc_insertion_point(module_scope)



from datetime import datetime
from enum import Enum, IntEnum
import inspect
import json
import sys
from typing import Any, Dict, List, Literal, Optional, Union
from typing_extensions import TypeAlias

from pydantic import BaseModel, Field, validator


class ApiResponse(BaseModel):
    retcode: int
    message: str = ""
    data: Any

    class Config:
        allow_population_by_field_name = True
        fields = {"message": {"alias": "msg"}}


class BotAuth(BaseModel):
    bot_id: str
    bot_secret: str


# http事件回调部分
# see https://webstatic.mihoyo.com/vila/bot/doc/callback.html
class CommandParam(BaseModel):
    desc: str


class Command(BaseModel):
    name: str
    desc: Optional[str] = None
    params: Optional[List[CommandParam]] = None


class TemplateCustomSettings(BaseModel):
    name: str
    url: str


class Template(BaseModel):
    id: str
    name: str
    desc: Optional[str] = None
    icon: str
    commands: Optional[List[Command]] = None
    custom_settings: Optional[List[TemplateCustomSettings]] = None
    is_allowed_add_to_other_villa: Optional[bool] = None


class Robot(BaseModel):
    villa_id: int
    template: Template


class QuoteMessage(BaseModel):
    content: str
    msg_uid: str
    bot_msg_id: Optional[str]
    send_at: int
    msg_type: str
    from_user_id: int
    from_user_nickname: str
    from_user_id_str: str


## 鉴权部分
## see https://webstatic.mihoyo.com/vila/bot/doc/auth_api/
class BotMemberAccessInfo(BaseModel):
    uid: int
    villa_id: int
    member_access_token: str
    bot_tpl_id: str


class CheckMemberBotAccessTokenReturn(BaseModel):
    access_info: BotMemberAccessInfo
    member: "Member"


# 大别野部分
# see https://webstatic.mihoyo.com/vila/bot/doc/villa_api/
class Villa(BaseModel):
    villa_id: int
    name: str
    villa_avatar_url: str
    onwer_uid: int
    is_official: bool
    introduce: str
    category_id: int
    tags: List[str]


# 用户部分
# see https://webstatic.mihoyo.com/vila/bot/doc/member_api/
class MemberBasic(BaseModel):
    uid: int
    nickname: str
    introduce: str
    avatar_url: str


class Member(BaseModel):
    basic: MemberBasic
    role_id_list: List[int]
    joined_at: datetime
    role_list: List["MemberRole"]


# 消息部分
# see https://webstatic.mihoyo.com/vila/bot/doc/message_api/
class MentionType(IntEnum):
    ALL = 1
    PART = 2

    def __repr__(self) -> str:
        return self.name


class MentionedRobot(BaseModel):
    type: Literal["mentioned_robot"] = Field(default="mentioned_robot", repr=False)
    bot_id: str

    bot_name: str = Field(exclude=True)


class MentionedUser(BaseModel):
    type: Literal["mentioned_user"] = Field(default="mentioned_user", repr=False)
    user_id: str

    user_name: Optional[str] = Field(exclude=True)


class MentionedAll(BaseModel):
    type: Literal["mention_all"] = Field(default="mention_all", repr=False)

    show_text: str = Field(exclude=True)


class VillaRoomLink(BaseModel):
    type: Literal["villa_room_link"] = Field(default="villa_room_link", repr=False)
    villa_id: str
    room_id: str

    room_name: Optional[str] = Field(exclude=True)


class Link(BaseModel):
    type: Literal["link"] = Field(default="link", repr=False)
    url: str
    requires_bot_access_token: bool

    show_text: str = Field(exclude=True)


class TextStyle(BaseModel):
    type: Literal["style"] = Field(default="style", repr=False)
    font_style: Literal["bold", "italic", "strikethrough", "underline"]


class TextEntity(BaseModel):
    offset: int
    length: int
    entity: Union[
        MentionedRobot,
        MentionedUser,
        MentionedAll,
        VillaRoomLink,
        Link,
        TextStyle,
    ]


class ImageSize(BaseModel):
    width: Optional[int] = None
    height: Optional[int] = None


class Image(BaseModel):
    url: str
    size: ImageSize = Field(default_factory=ImageSize)
    file_size: Optional[int] = None


class PreviewLink(BaseModel):
    icon_url: str
    image_url: str
    is_internal_link: bool
    title: str
    content: str
    url: str
    source_name: str


class Badge(BaseModel):
    icon_url: str
    text: str
    url: str


class PostMessageContent(BaseModel):
    post_id: str

    @validator("post_id")
    @classmethod
    def __deal_post_id(cls, v: str):
        s = v.split("/")[-1]
        if s.isdigit():
            return s
        raise ValueError(f"Invalid post_id: {v}, post_id must be a number.")


class TextMessageContent(BaseModel):
    text: str
    entities: List[TextEntity] = Field(default_factory=list)
    images: Optional[List[Image]] = None
    preview_link: Optional[PreviewLink] = None
    badge: Optional[Badge] = None


ImageMessageContent: TypeAlias = Image


class MentionedInfo(BaseModel):
    type: MentionType
    user_id_list: List[str] = Field(default_factory=list, alias="userIdList")


class QuoteInfo(BaseModel):
    quoted_message_id: str
    quoted_message_send_time: int
    original_message_id: str
    original_message_send_time: int


class User(BaseModel):
    portrait_uri: str = Field(alias="portraitUri")
    extra: Dict[str, Any]
    name: str
    alias: str
    id: str
    portrait: str

    @validator("extra", pre=True)
    @classmethod
    def extra_str_to_dict(cls, v: Any):
        if isinstance(v, str):
            return json.loads(v)
        return v


class Component(BaseModel):
    id: str
    text: str
    type: int = 1
    need_callback: Optional[bool] = None
    extra: str = ""


ComponentGroup = List[Component]


class Panel(BaseModel):
    template_id: Optional[int] = None
    small_component_group_list: Optional[List[ComponentGroup]] = None
    mid_component_group_list: Optional[List[ComponentGroup]] = None
    big_component_group_list: Optional[List[ComponentGroup]] = None


class ButtonType(IntEnum):
    Callback = 1
    Input = 2
    Link = 3


class Button(Component):
    c_type: ButtonType
    input: Optional[str] = None
    link: Optional[str] = None
    need_token: Optional[bool] = None


class CallbackButton(Button):
    c_type: Literal[ButtonType.Callback] = Field(
        default=ButtonType.Callback,
        init=False,
    )
    need_callback: Literal[True] = True


class InputButton(Button):
    c_type: Literal[ButtonType.Input] = Field(default=ButtonType.Input, init=False)
    input: str


class LinkButton(Button):
    c_type: Literal[ButtonType.Link] = Field(default=ButtonType.Link, init=False)
    link: str
    need_token: bool


class Trace(BaseModel):
    visual_room_version: str
    app_version: str
    action_type: int
    bot_msg_id: str
    client: str
    env: str
    rong_sdk_version: str


class MessageContentInfo(BaseModel):
    content: Union[TextMessageContent, ImageMessageContent, PostMessageContent]
    mentioned_info: Optional[MentionedInfo] = Field(default=None, alias="mentionedInfo")
    quote: Optional[QuoteInfo] = None
    panel: Optional[Panel] = None


class MessageContentInfoGet(MessageContentInfo):
    user: User
    trace: Optional[Trace] = None


# 房间部分
# see https://webstatic.mihoyo.com/vila/bot/doc/room_api/
class ListRoom(BaseModel):
    room_id: int
    room_name: str
    room_type: "RoomType"
    group_id: int


class Room(ListRoom):
    room_default_notify_type: "RoomDefaultNotifyType"
    send_msg_auth_range: "SendMsgAuthRange"


class RoomType(str, Enum):
    CHAT = "BOT_PLATFORM_ROOM_TYPE_CHAT_ROOM"
    POST = "BOT_PLATFORM_ROOM_TYPE_POST_ROOM"
    SCENE = "BOT_PLATFORM_ROOM_TYPE_SCENE_ROOM"
    LIVE = "BOT_PLATFORM_ROOM_TYPE_LIVE_ROOM"
    INVALID = "BOT_PLATFORM_ROOM_TYPE_INVALID"

    def __repr__(self) -> str:
        return self.name


class RoomDefaultNotifyType(str, Enum):
    NOTIFY = "BOT_PLATFORM_DEFAULT_NOTIFY_TYPE_NOTIFY"
    IGNORE = "BOT_PLATFORM_DEFAULT_NOTIFY_TYPE_IGNORE"
    INVALID = "BOT_PLATFORM_DEFAULT_NOTIFY_TYPE_INVALID"

    def __repr__(self) -> str:
        return self.name


class SendMsgAuthRange(BaseModel):
    is_all_send_msg: bool
    roles: List[int]


class GroupRoom(BaseModel):
    group_id: int
    group_name: str
    room_list: List[ListRoom]


class ListRoomType(IntEnum):
    CHAT = 1
    POST = 2
    SCENE = 3

    def __repr__(self) -> str:
        return self.name


class CreateRoomType(IntEnum):
    CHAT = 1
    POST = 2
    SCENE = 3

    def __repr__(self) -> str:
        return self.name


class CreateRoomDefaultNotifyType(IntEnum):
    NOTIFY = 1
    IGNORE = 2

    def __repr__(self) -> str:
        return self.name


class Group(BaseModel):
    group_id: int
    group_name: str


# 身份组部分
# see https://webstatic.mihoyo.com/vila/bot/doc/role_api/
class MemberRole(BaseModel):
    id: int
    name: str
    color: str
    role_type: "RoleType"
    villa_id: int
    member_num: int
    web_color: str
    font_color: str
    bg_color: str
    has_manage_perm: Optional[bool] = None
    is_all_room: bool
    room_ids: List[int]
    color_scheme_id: int
    priority: int
    permissions: Optional[List["Permission"]] = None


class PermissionDetail(BaseModel):
    key: str
    name: str
    describe: str


class RoleType(str, Enum):
    ALL_MEMBER = "MEMBER_ROLE_TYPE_ALL_MEMBER"
    ADMIN = "MEMBER_ROLE_TYPE_ADMIN"
    OWNER = "MEMBER_ROLE_TYPE_OWNER"
    CUSTOM = "MEMBER_ROLE_TYPE_CUSTOM"
    UNKNOWN = "MEMBER_ROLE_TYPE_UNKNOWN"

    def __repr__(self) -> str:
        return self.name


class Permission(str, Enum):
    MENTION_ALL = "mention_all"
    RECALL_MESSAGE = "recall_message"
    PIN_MESSAGE = "pin_message"
    MANAGE_MEMBER_ROLE = "manage_member_role"
    EDIT_VILLA_INFO = "edit_villa_info"
    MANAGE_GROUP_AND_ROOM = "manage_group_and_room"
    VILLA_SILENCE = "villa_silence"
    BLACK_OUT = "black_out"
    HANDLE_APPLY = "handle_apply"
    MANAGE_CHAT_ROOM = "manage_chat_room"
    VIEW_DATA_BOARD = "view_data_board"
    MANAGE_CUSTOM_EVENT = "manage_custom_event"
    LIVE_ROOM_ORDER = "live_room_order"
    MANAGE_SPOTLIGHT_COLLECTION = "manage_spotlight_collection"

    def __repr__(self) -> str:
        return self.name


class Color(str, Enum):
    GREY = "#6173AB"
    PINK = "#F485D8"
    RED = "#F47884"
    ORANGE = "#FFA54B"
    GREEN = "#7ED321"
    BLUE = "#59A1EA"
    PURPLE = "#977EE1"
    LIGHT_BLUE = "#8F9BBF"  # 此颜色为所有人身份组颜色，无法作为创建和编辑身份组的颜色


# 表态表情部分
# see https://webstatic.mihoyo.com/vila/bot/doc/emoticon_api/
class Emoticon(BaseModel):
    emoticon_id: int
    describe_text: str
    icon: str


# 审核部分
# see https://webstatic.mihoyo.com/vila/bot/doc/audit_api/audit.html
class ContentType(str, Enum):
    TEXT = "AuditContentTypeText"
    IMAGE = "AuditContentTypeImage"


# 图片部分
# see https://webstatic.mihoyo.com/vila/bot/doc/img_api/upload.html


class CallbackVar(BaseModel):
    x_extra: str = Field(alias="x:extra")


class ImageUploadParams(BaseModel):
    accessid: str
    callback: str
    callback_var: CallbackVar
    dir: str
    expire: str
    host: str
    name: str
    policy: str
    signature: str
    x_oss_content_type: str
    object_acl: str
    content_disposition: str
    key: str
    success_action_status: str

    def to_upload_data(self) -> Dict[str, Any]:
        return {
            "x:extra": self.callback_var.x_extra,
            "OSSAccessKeyId": self.accessid,
            "signature": self.signature,
            "success_action_status": self.success_action_status,
            "name": self.name,
            "callback": self.callback,
            "x-oss-content-type": self.x_oss_content_type,
            "key": self.key,
            "policy": self.policy,
            "Content-Disposition": self.content_disposition,
        }


class UploadImageParamsReturn(BaseModel):
    type: str
    file_name: str
    max_file_size: int
    params: ImageUploadParams


class ImageUploadResult(BaseModel):
    object: str
    secret_url: str
    url: str


# Websocket 部分
# see https://webstatic.mihoyo.com/vila/bot/doc/websocket/websocket_api.html
class WebsocketInfo(BaseModel):
    websocket_url: str
    uid: int
    app_id: int
    platform: int
    device_id: str


for _, obj in inspect.getmembers(sys.modules[__name__]):
    if inspect.isclass(obj) and issubclass(obj, BaseModel):
        obj.update_forward_refs()


__all__ = [
    "ApiResponse",
    "BotAuth",
    "Command",
    "Template",
    "Robot",
    "QuoteMessage",
    "BotMemberAccessInfo",
    "CheckMemberBotAccessTokenReturn",
    "Villa",
    "MemberBasic",
    "Member",
    "MentionType",
    "MentionedRobot",
    "MentionedUser",
    "MentionedAll",
    "VillaRoomLink",
    "Link",
    "TextStyle",
    "TextEntity",
    "TextMessageContent",
    "ImageMessageContent",
    "PostMessageContent",
    "MentionedInfo",
    "QuoteInfo",
    "User",
    "Component",
    "Panel",
    "ButtonType",
    "Button",
    "CallbackButton",
    "InputButton",
    "LinkButton",
    "Trace",
    "ImageSize",
    "Image",
    "PreviewLink",
    "Badge",
    "MessageContentInfo",
    "MessageContentInfoGet",
    "Room",
    "RoomType",
    "RoomDefaultNotifyType",
    "SendMsgAuthRange",
    "GroupRoom",
    "ListRoomType",
    "CreateRoomType",
    "CreateRoomDefaultNotifyType",
    "Group",
    "MemberRole",
    "PermissionDetail",
    "RoleType",
    "Permission",
    "Color",
    "Emoticon",
    "ContentType",
    "ImageUploadParams",
    "UploadImageParamsReturn",
    "WebsocketInfo",
]
